# Bu araç @keyiflerolsun tarafından | @KekikAkademi için yazılmıştır.

import os, sys

def terminal_baslik(mesaj:str):
    if os.name == "nt":
        import ctypes
        ctypes.windll.kernel32.SetConsoleTitleW(mesaj)
        return

    if "KONSOLE_VERSION" in os.environ:
        # https://stackoverflow.com/questions/19897787/change-konsole-tab-title-from-command-line-and-make-it-persistent
        # TODO this works only once per stage where it's updated at the very beginning
        sys.stdout.write(f"\033]30;{mesaj}\007")
    else:
        # https://stackoverflow.com/questions/25872409/set-gnome-terminal-window-title-in-python/47262154#47262154
        sys.stdout.write(f"\33]0;{mesaj}\a")

    sys.stdout.flush()


# Bu araç @keyiflerolsun tarafından | @KekikAkademi için yazılmıştır.
# ! https://github.com/Eralde/keenetic-dark-theme-extension/blob/master/app/scripts/uiExtension/routesToolbar/routes-tools.service.js

from Kekik.cli import konsol
from httpx     import Client, HTTPError

class Domain2IP:
    def __init__(self, domain: str):
        self.oturum = Client(headers={"accept": "application/dns-json"})
        self.domain = domain

        self.CORS_PROXY_URL = "https://ts-cors-proxy.eralde.workers.dev"
        self.CF_DOH_URL     = "https://cloudflare-dns.com/dns-query"

    def _dns_sorgula(self, domain: str, sorgu_turu: str):
        if not domain:
            domain = self.domain

        try:
            url   = f"{self.CORS_PROXY_URL}/{self.CF_DOH_URL}?name={domain}&type={sorgu_turu}"
            yanit = self.oturum.get(url)
            yanit.raise_for_status()

            return yanit.json().get("Answer" if sorgu_turu == "A" else "Authority", [])

        except HTTPError as http_hatasi:
            konsol.log(f"[red][!] HTTP hatası meydana geldi: {http_hatasi}")

        except Exception as diger_hata:
            konsol.log(f"[red][!] Başka bir hata meydana geldi: {diger_hata}")

        return []

    def a_kayitlari(self, domain: str = None):
        return self._dns_sorgula(domain, "A")

    def cname_kayitlari(self, domain: str = None):
        return self._dns_sorgula(domain, "CNAME")

    def tum_ip_adresleri(self):
        ip_adresleri         = set()
        sorgulanan_domainler = set()

        def ipleri_ekle(kayitlar):
            for kayit in kayitlar:
                kayit_tipi   = kayit.get("type")
                kayit_verisi = kayit.get("data")
                match kayit_tipi:
                    case 1:  # A kaydı
                        ip_adresleri.add(kayit_verisi)
                    case 5:  # CNAME kaydı
                        alt_domain = kayit_verisi.rstrip(".")
                        if alt_domain not in sorgulanan_domainler:
                            sorgulanan_domainler.add(alt_domain)
                            ipleri_ekle(self.a_kayitlari(alt_domain))

        ipleri_ekle(self.a_kayitlari())
        ipleri_ekle(self.cname_kayitlari())
        return sorted(list(ip_adresleri))

    def ip_to_binary(self, ip_str):
        return "".join([f"{int(octet):08b}" for octet in ip_str.split(".")])

    def binary_to_ip(self, binary_str):
        octets = [
            str(int(binary_str[i : i + 8], 2)) for i in range(0, len(binary_str), 8)
        ]

        return ".".join(octets)

    def get_mask_binary(self, ones_length):
        return "".join(["1" if i < ones_length else "0" for i in range(32)])

    def longest_common_prefix(self, a, b):
        i = 0
        while i < min(len(a), len(b)) and a[i] == b[i]:
            i += 1

        return i

    def grubu_isle(self, grup):
        if len(grup) == 1:
            return None
            # return {
            #     "subnet" : f"{self.binary_to_ip(grup[0])}/32",
            #     "ipler"  : [self.binary_to_ip(grup[0])],
            # }

        lcp       = self.longest_common_prefix(grup[0], grup[-1])
        subnet_ip = self.binary_to_ip(grup[0][:lcp] + "0" * (32 - lcp))

        return {
            "subnet" : f"{subnet_ip}/{lcp}",
            "ipler"  : sorted([self.binary_to_ip(ip) for ip in grup]),
        }

    def subnetlere_ayir(self, ip_listesi, min_ortak_bit=24):
        if not ip_listesi:
            konsol.log("[red][!] Subnetlere ayırmak için IP yok.")
            return []

        binary_listesi = sorted(map(self.ip_to_binary, ip_listesi))
        if not binary_listesi:
            konsol.log("[red][!] İşlenecek binary IP verisi yok.")
            return []

        gruplar     = []
        mevcut_grup = [binary_listesi[0]]
        for binary_ip in binary_listesi[1:]:
            if self.longest_common_prefix(mevcut_grup[-1], binary_ip) < min_ortak_bit:
                gruplar.append(mevcut_grup)
                mevcut_grup = [binary_ip]
            else:
                mevcut_grup.append(binary_ip)

        gruplar.append(mevcut_grup)

        return [self.grubu_isle(grup) for grup in gruplar if self.grubu_isle(grup)]

    @property
    def bilgi(self):
        tum_ipler = self.tum_ip_adresleri()
        subnetler = [subnet["subnet"] for subnet in self.subnetlere_ayir(tum_ipler) if subnet]

        return {
            "domain"    : self.domain,
            "ipler"     : tum_ipler or None,
            "subnetler" : subnetler or None,
        }

from sys import argv

def domain_sorgu():
    print()

    if len(argv) != 2:
        konsol.print("[bold yellow2][!] Lütfen Domain Girin..")
        konsol.print("\n[turquoise2]Örn.: [pale_green1]Domain2IP keyiflerolsun.dev\n")
        return

    konsol.print(Domain2IP(argv[1]).bilgi)

    print()


# Bu araç @keyiflerolsun tarafından | @KekikAkademi için yazılmıştır.

import os, json

def dict2json(sozluk:dict, liste_key:str, dosya_adi:str) -> bool:
    """
    Verilen `sözlük` tipli veriyi ilgili `anahtar`a göre `hedef dosya`ya yazar.

    Parametreler
    ------------
        >>> sozluk    (dict) # Yazılması İstenen Veri
        >>> liste_key (str)  # Sıralanması İstenen Anahtar
        >>> dosya_adi (str)  # Yazılması İstenen Json Dosya Yolu

    Kullanım
    --------
    ```
    dict2json(
        sozluk    = {
            'id'            : 1,
            'kullanici_adi' : "@keyiflerolsun",
            'ad_soyad'      : "Ömer Faruk Sancak"
        },
        liste_key = "kullanici_id",
        dosya_adi = "kullanicilar.json"
    )
    ```

    Dönüt
    -----
        >>> bool
    """
    if os.path.isfile(dosya_adi):
        with open(dosya_adi, encoding="utf-8") as gelen_json:
            gelen_veri    = json.load(gelen_json)
            gelen_kisiler = [kisi[liste_key] for kisi in gelen_veri]

        if sozluk[liste_key] not in gelen_kisiler:
            gelen_veri.append(sozluk)
            gelen_essiz = [dict(sozluk) for sozluk in {tuple(liste_ici.items()) for liste_ici in gelen_veri}]
            gelen_a_z   = sorted(gelen_essiz, key=lambda sozluk: sozluk[liste_key])

            with open(dosya_adi, mode="w", encoding="utf-8") as dosya:
                dosya.write(json.dumps(gelen_a_z, indent=2, ensure_ascii=False, sort_keys=False))

            return True

        return False

    else:
        with open(dosya_adi, mode="w", encoding="utf-8") as dosya:
            liste = [sozluk]
            essiz = [dict(sozluk) for sozluk in {tuple(liste_ici.items()) for liste_ici in liste}]
            a_z   = sorted(essiz, key=lambda sozluk: sozluk[liste_key])

            dosya.write(json.dumps(a_z, indent=2, ensure_ascii=False, sort_keys=False))

            return True


# Bu araç @keyiflerolsun tarafından | @KekikAkademi için yazılmıştır.

from os         import name as sistem
from subprocess import check_output, STDOUT
from requests   import get
from uuid       import uuid5, NAMESPACE_DNS

def str2uuid(metin) -> str:
    return str(uuid5(NAMESPACE_DNS, str(metin))).upper()

def benim_hwid() -> str:
    # HWID Yakala
    if sistem == "nt":
        benim_hwid = str2uuid(check_output("wmic csproduct get uuid").decode().split("\n")[1])
    else:
        try:
            from GPUtil import getGPUs
        except ModuleNotFoundError:
            from os import system
            system("pip3 install gputil")
            system("clear")
            from GPUtil import getGPUs

        if ekran_kartlari := getGPUs():
            benim_hwid = str2uuid(ekran_kartlari[0].uuid[4:])
        else:
            try:
                benim_hwid = str2uuid(check_output(["cat", "/var/lib/dbus/machine-id"], stderr=STDOUT).decode())
            except Exception:
                benim_hwid = str2uuid(check_output("lscpu").decode() + check_output(["uname", "-a"]).decode())

    return benim_hwid

def hwid_kontrol(kontrol_url:str) -> bool:
    # Çevrimiçi HWID Kontrolü
    return benim_hwid() in get(kontrol_url).text


# Bu araç @keyiflerolsun tarafından | @KekikAkademi için yazılmıştır.

from os.path import isfile

def dosya2set(dosya_yolu:str) -> set[str] | None:
    try:
        return {satir.strip().replace("\n", "") for satir in open(dosya_yolu, "r", encoding="utf-8") if satir.strip()} if isfile(dosya_yolu) else None
    except Exception:
        return None


# -*- coding: utf8 -*-

try:
    __instance__ = unicode
except:
    __instance__ = str


class unicode_tr(__instance__):
    CHARMAP = {
        "to_upper": {
            u"ı": u"I",
            u"i": u"İ",
        },
        "to_lower": {
            u"I": u"ı",
            u"İ": u"i",
        }
    }

    def lower(self):
        for key, value in self.CHARMAP.get("to_lower").items():
            self = self.replace(key, value)

        return unicode_tr(getattr(__instance__, "lower")(self))

    def upper(self):
        for key, value in self.CHARMAP.get("to_upper").items():
            self = self.replace(key, value)

        return unicode_tr(getattr(__instance__, "upper")(self))

    def replace(self, *args, **kwargs):
        return unicode_tr(getattr(__instance__, "replace")(self, args[0], args[1]))

    def capitalize(self):
        first, rest = self[0], self[1:]
        return unicode_tr(unicode_tr(first).upper() + unicode_tr(rest).lower())

    def title(self):
        return unicode_tr(" ".join(map(lambda x: unicode_tr(x).capitalize(), self.split())).title())


# Bu araç @keyiflerolsun tarafından | @KekikAkademi için yazılmıştır.

from typing       import Optional
from requests     import head, get
from urllib.parse import unquote
from tqdm         import tqdm

def indirilebilir_mi(url:str) -> bool:
    istek  = head(url, allow_redirects=True)
    header = istek.headers
    content_type = header.get("content-type")

    if "text" in content_type.lower():
        return False

    return "html" not in content_type.lower()

def dosya_indir(url:str, dosya_adi:str=None) -> Optional[str]:
    kontrol = indirilebilir_mi(url)
    if not kontrol:
        print(f"HATA!, İndirilebilir Değil! » {url}")
        return None

    if not dosya_adi:
        dosya_adi = unquote(url.split("/")[-1])
    elif "." not in dosya_adi:
        uzanti = unquote(url.split("/")[-1]).split(".")[-1]
        dosya_adi = f"{dosya_adi}.{uzanti}"

    istek = get(url, stream=True)
    dosya_boyutu = int(istek.headers.get("content-length", 0))

    bar_format   = "{l_bar} [{rate_fmt}] | {bar}| [{n_fmt}B / {total_fmt}B] » [{elapsed} / {remaining}]"
    progress_bar = tqdm(desc=dosya_adi, total=dosya_boyutu, unit="B", unit_scale=True, bar_format=bar_format)
    with open(dosya_adi, "wb") as dosya:
        for veri in istek.iter_content(1024): # 1KB
            progress_bar.update(len(veri))
            dosya.write(veri)
    progress_bar.close()

    if dosya_boyutu != 0 and progress_bar.n != dosya_boyutu:
        print(f"HATA!, Bişeyler Yanlış Gitti! » {dosya_adi}")
        return None

    return dosya_adi